<?php

namespace Drupal\images_optimizer;

use Drupal\Core\File\FileSystemInterface;
use Drupal\file\FileInterface;
use Drupal\images_optimizer\Optimizer\OptimizerInterface;
use Drupal\images_optimizer\Optimizer\OptimizerServiceCollector;
use Psr\Log\LoggerAwareTrait;

/**
 * Handler for files.
 *
 * @package Drupal\images_optimizer
 */
class ImagesOptimizerFileHandler {

  use LoggerAwareTrait;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  private $fileSystem;

  /**
   * The optimizer service collector.
   *
   * @var \Drupal\images_optimizer\Optimizer\OptimizerServiceCollector
   */
  private $optimizerServiceCollector;

  /**
   * ImagesOptimizerFileHandler constructor.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   * @param \Drupal\images_optimizer\Optimizer\OptimizerServiceCollector $optimizer_service_collector
   *   The optimizer service collector.
   */
  public function __construct(FileSystemInterface $file_system, OptimizerServiceCollector $optimizer_service_collector) {
    $this->fileSystem = $file_system;
    $this->optimizerServiceCollector = $optimizer_service_collector;
  }

  /**
   * Handle a file and try to optimize it.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file.
   *
   * @return bool
   *   TRUE if the file was optimized, FALSE otherwise.
   */
  public function handle(FileInterface $file) {
    if (!empty(file_validate_is_image($file))) {
      return FALSE;
    }

    $image_path = $this->fileSystem->realpath($file->getFileUri());
    if (FALSE === $image_path) {
      $this->logger->error(sprintf('Could not resolve the absolute path of the image (URI "%s").', $file->getFileUri()));

      return FALSE;
    }

    $optimizer = $this->optimizerServiceCollector->get($file->getMimeType());
    if (!$optimizer instanceof OptimizerInterface) {
      return FALSE;
    }

    if (!$optimizer->optimize($image_path, $file)) {
      $this->logger->error(sprintf('The image optimization failed (optimizer "%s", image path "%s").', get_class($optimizer), $image_path));

      return FALSE;
    }

    return TRUE;
  }

}
