<?php

namespace Drupal\images_optimizer\Optimizer;

use Psr\Log\LoggerAwareTrait;

/**
 * Service collector for the optimizers.
 *
 * @package Drupal\images_optimizer
 */
class OptimizerServiceCollector {

  use LoggerAwareTrait;

  /**
   * The unprocessed array of optimizers.
   *
   * @var array
   */
  private $unprocessedOptimizers;

  /**
   * The processed array of optimizers.
   *
   * @var \Drupal\images_optimizer\Optimizer\OptimizerInterface[]|null
   */
  private $processedOptimizers;

  /**
   * OptimizerServiceCollector constructor.
   */
  public function __construct() {
    $this->unprocessedOptimizers = [];
    $this->processedOptimizers = NULL;
  }

  /**
   * Get the optimizer for a mime type.
   *
   * @param string $mime_type
   *   The mime type.
   *
   * @return \Drupal\images_optimizer\Optimizer\OptimizerInterface|null
   *   The matching optimizer, NULL if there is none.
   */
  public function get($mime_type) {
    if (empty($this->unprocessedOptimizers)) {
      return NULL;
    }

    if (NULL === $this->processedOptimizers) {
      $this->process();
    }

    if (!isset($this->processedOptimizers[$mime_type])) {
      return NULL;
    }

    return $this->processedOptimizers[$mime_type];
  }

  /**
   * Add an optimizer.
   *
   * @param \Drupal\images_optimizer\Optimizer\OptimizerInterface $optimizer
   *   The optimizer.
   * @param int $priority
   *   The priority. A higher priority will be used over a lower priority.
   */
  public function add(OptimizerInterface $optimizer, $priority = 0) {
    $this->processedOptimizers = NULL;

    foreach (array_unique($optimizer->getSupportedMimeTypes()) as $mime_type) {
      if (!isset($this->unprocessedOptimizers[$mime_type])) {
        $this->unprocessedOptimizers[$mime_type] = [];
      }

      // If two optimizers support the same mime type and have the same
      // priority, the last one to be added will be used.
      $this->unprocessedOptimizers[$mime_type][$priority] = $optimizer;
    }
  }

  /**
   * Process optimizers by their priority.
   */
  private function process() {
    foreach ($this->unprocessedOptimizers as &$optimizers) {
      krsort($optimizers);
    }

    unset($optimizers);

    $this->processedOptimizers = array_map('reset', $this->unprocessedOptimizers);
  }

}
