<?php

namespace Drupal\Tests\images_optimizer\Functional;

use Drupal\Core\File\FileSystemInterface;
use Drupal\file\Entity\File;
use Drupal\images_optimizer\ImagesOptimizerFileHandler;
use Drupal\images_optimizer\Optimizer\OptimizerInterface;
use Drupal\images_optimizer\Optimizer\OptimizerServiceCollector;
use Drupal\KernelTests\KernelTestBase;
use Symfony\Component\HttpKernel\Tests\Logger;

/**
 * Kernel test class for the ImagesOptimizerFileHandler class.
 *
 * @package Drupal\Tests\images_optimizer\Functional
 */
class ImagesOptimizerFileHandlerTest extends KernelTestBase {

  /**
   * An image that exists.
   *
   * @var string
   */
  const VALID_IMAGE_URI = 'core/misc/druplicon.png';

  /**
   * A file that exists but that is not an image.
   *
   * @var string
   */
  const EXISTING_FILE_THAT_IS_NOT_AN_IMAGE_URI = 'core/misc/drupal.js';

  /**
   * {@inheritdoc}
   */
  public static $modules = ['system', 'user', 'file'];

  /**
   * The optimizer service collector.
   *
   * @var \Drupal\images_optimizer\Optimizer\OptimizerServiceCollector
   */
  private $optimizerServiceCollector;

  /**
   * The logger.
   *
   * @var \Symfony\Component\HttpKernel\Tests\Logger
   */
  private $logger;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  private $fileSystem;

  /**
   * The images optimizer file handler to test.
   *
   * @var \Drupal\images_optimizer\ImagesOptimizerFileHandler
   */
  private $imagesOptimizerFileHandler;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->optimizerServiceCollector = new OptimizerServiceCollector();

    $this->logger = new Logger();

    $this->fileSystem = $this->container->get('file_system');

    $this->imagesOptimizerFileHandler = $this->getImagesOptimizerFileHandler($this->fileSystem);
  }

  /**
   * Test handle() when the file is not a valid image.
   *
   * @dataProvider handleWhenTheFileIsNotValidImageProvider
   */
  public function testHandleWhenTheFileIsNotValidImage($fileUri) {
    $file = File::create();
    $file->setFileUri($fileUri);

    $this->assertFalse($this->imagesOptimizerFileHandler->handle($file));
  }

  /**
   * Provide the data for the testHandleWhenTheFileIsNotAValidImage() method.
   *
   * @return array
   *   The data.
   */
  public function handleWhenTheFileIsNotValidImageProvider() {
    // The first one is for a file that has a valid image extension but that
    // does not exists.
    return [
      ['foo/bar/qux.png'],
      [self::EXISTING_FILE_THAT_IS_NOT_AN_IMAGE_URI],
    ];
  }

  /**
   * Test handle() when the absolute image path cannot be resolved.
   */
  public function testHandleWhenTheAbsoluteImagePathCannotBeResolved() {
    $fileSystem = $this->createMock(FileSystemInterface::class);
    $fileSystem
      ->expects($this->atLeastOnce())
      ->method('realpath')
      ->with(self::VALID_IMAGE_URI)
      ->willReturn(FALSE);

    $imagesOptimizerFileHandler = $this->getImagesOptimizerFileHandler($fileSystem);

    $this->assertFalse($imagesOptimizerFileHandler->handle($this->getValidFile()));

    $logs = $this->logger->getLogs('error');
    $this->assertCount(1, $logs);
    $this->assertSame(sprintf('Could not resolve the absolute path of the image (URI "%s").', self::VALID_IMAGE_URI), reset($logs));
  }

  /**
   * Test handle().
   *
   * When there is no optimizer configured for the image mime type.
   */
  public function testHandleWhenThereIsNoOptimizerConfiguredForTheImageMimeType() {
    $this->assertFalse($this->imagesOptimizerFileHandler->handle($this->getValidFile()));
  }

  /**
   * Test handle() when the optimization failed.
   */
  public function testHandleWhenTheOptimizationFailed() {
    list($file, $image_path, $optimizer) = $this->prepareHandleOptimize(FALSE);

    $this->assertFalse($this->imagesOptimizerFileHandler->handle($file));

    $logs = $this->logger->getLogs('error');
    $this->assertCount(1, $logs);
    $this->assertSame(sprintf('The image optimization failed (optimizer "%s", image path "%s").', get_class($optimizer), $image_path), reset($logs));
  }

  /**
   * Test handle() when the optimization was successful.
   */
  public function testHandleWhenTheOptimizationWasSuccessful() {
    list($file) = $this->prepareHandleOptimize(TRUE);

    $this->assertTrue($this->imagesOptimizerFileHandler->handle($file));
  }

  /**
   * Get a images optimizer file handler.
   *
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system.
   *
   * @return \Drupal\images_optimizer\ImagesOptimizerFileHandler
   *   The images optimizer file handle.
   */
  private function getImagesOptimizerFileHandler(FileSystemInterface $fileSystem) {
    $imagesOptimizerFileHandler = new ImagesOptimizerFileHandler($fileSystem, $this->optimizerServiceCollector);
    $imagesOptimizerFileHandler->setLogger($this->logger);

    return $imagesOptimizerFileHandler;
  }

  /**
   * Get a valid file.
   *
   * @return \Drupal\file\Entity\File
   *   The file.
   */
  private function getValidFile() {
    $file = File::create();
    $file->setFileUri(self::VALID_IMAGE_URI);
    $file->setMimeType('image/png');

    return $file;
  }

  /**
   * Prepare the context for the handle() method test.
   *
   * When it actually reaches the optimize() call.
   *
   * @param bool $successful
   *   TRUE if the optimization should be successful, FALSE otherwise.
   *
   * @return array
   *   An array containing the file, the image path and the optimizer.
   */
  private function prepareHandleOptimize($successful) {
    $file = $this->getValidFile();
    $image_path = $this->fileSystem->realpath(self::VALID_IMAGE_URI);

    $optimizer = $this->createMock(OptimizerInterface::class);
    $optimizer
      ->expects($this->any())
      ->method('getSupportedMimeTypes')
      ->willReturn(['image/png']);
    $optimizer
      ->expects($this->atLeastOnce())
      ->method('optimize')
      ->with($image_path, $file)
      ->willReturn($successful);

    $this->optimizerServiceCollector->add($optimizer, 0);

    return [$file, $image_path, $optimizer];
  }

}
