<?php

namespace Drupal\Tests\images_optimizer\Unit\Optimizer;

use Drupal\images_optimizer\Optimizer\OptimizerInterface;
use Drupal\images_optimizer\Optimizer\OptimizerServiceCollector;
use Drupal\Tests\UnitTestCase;

/**
 * Unit test class for the OptimizerServiceCollector class.
 *
 * @package Drupal\Tests\images_optimizer\Optimizer\Unit
 */
class OptimizerServiceCollectorTest extends UnitTestCase {

  /**
   * The optimizer service collector to test.
   *
   * @var \Drupal\images_optimizer\Optimizer\OptimizerServiceCollector
   */
  private $optimizerServiceCollector;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    $this->optimizerServiceCollector = new OptimizerServiceCollector();
  }

  /**
   * Test get() when there is no optimizers.
   */
  public function testGetWhenThereIsNoOptimizers() {
    $this->assertNull($this->optimizerServiceCollector->get('image/png'));
  }

  /**
   * Test get().
   *
   * When there is no optimizer configured for the requested mime type.
   */
  public function testGetWhenThereIsNoOptimizerConfiguredForTheRequestedMimeType() {
    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 0);

    $this->assertNull($this->optimizerServiceCollector->get('image/jpeg'));
  }

  /**
   * Test get().
   *
   * When there is only one optimizer configured for the requested mime type.
   */
  public function testGetWhenThereIsOnlyOneOptimizerConfiguredForTheRequestedMimeType() {
    $optimizer = $this->getOptimizer(['image/png']);

    $this->optimizerServiceCollector->add($optimizer, 0);

    $this->assertSame($optimizer, $this->optimizerServiceCollector->get('image/png'));
  }

  /**
   * Test get().
   *
   * When there are several optimizers configured for the requested mime type.
   */
  public function testGetWhenThereAreSeveralOptimizersConfiguredForTheRequestedMimeType() {
    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 0);

    $optimizer2 = $this->getOptimizer(['image/png']);
    $this->optimizerServiceCollector->add($optimizer2, 255);

    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 128);

    $this->assertSame($optimizer2, $this->optimizerServiceCollector->get('image/png'));
  }

  /**
   * Test get().
   *
   * When an optimizer configured for the requested mime type is added later
   * with a higher priority.
   */
  public function testGetWhenAnOptimizerConfiguredForTheRequestedMimeTypeIsAddedLaterWithHigherPriority() {
    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 0);
    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 128);

    $this->optimizerServiceCollector->get('image/png');

    $optimizer3 = $this->getOptimizer(['image/png']);
    $this->optimizerServiceCollector->add($optimizer3, 255);

    $this->assertSame($optimizer3, $this->optimizerServiceCollector->get('image/png'));
  }

  /**
   * Test get().
   *
   * When an optimizer configured for the requested mime type is added later
   * with a lower priority.
   */
  public function testGetWhenAnOptimizerConfiguredForTheRequestedMimeTypeIsAddedLaterWithLowerPriority() {
    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 0);
    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 128);

    $this->optimizerServiceCollector->get('image/png');

    $optimizer3 = $this->getOptimizer(['image/png']);
    $this->optimizerServiceCollector->add($optimizer3, 64);

    $this->assertNotSame($optimizer3, $this->optimizerServiceCollector->get('image/png'));
  }

  /**
   * Test get().
   *
   * When an optimizer configured for the requested mime type is added later
   * with the same priority.
   */
  public function testGetWhenAnOptimizerConfiguredForTheRequestedMimeTypeIsAddedLaterWithTheSamePriority() {
    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 0);
    $this->optimizerServiceCollector->add($this->getOptimizer(['image/png']), 128);

    $this->optimizerServiceCollector->get('image/png');

    $optimizer3 = $this->getOptimizer(['image/png']);
    $this->optimizerServiceCollector->add($optimizer3, 128);

    $this->assertSame($optimizer3, $this->optimizerServiceCollector->get('image/png'));
  }

  /**
   * Get a mocked optimizer.
   *
   * @param array $supportedMimeTypes
   *   The supported mime types (array of string).
   *
   * @return \PHPUnit_Framework_MockObject_MockObject
   *   The mocked optimizer.
   */
  private function getOptimizer(array $supportedMimeTypes) {
    $optimizer = $this->createMock(OptimizerInterface::class);
    $optimizer
      ->expects($this->any())
      ->method('getSupportedMimeTypes')
      ->willReturn($supportedMimeTypes);

    return $optimizer;
  }

}
